==> Botan-1.1.10.tgz: Botan-1.1.10/src/ssl3_mac.cpp <==
/*************************************************
* SSL3-MAC Source File                           *
* (C) 1999-2003 The Botan Project                *
*************************************************/

#include <botan/ssl3_mac.h>
#include <botan/lookup.h>

namespace Botan {

/*************************************************
* Update a SSL3-MAC Calculation                  *
*************************************************/
void SSL3_MAC::add_data(const byte input[], u32bit length)
   {
   hash->update(input, length);
   }

/*************************************************
* Finalize a SSL3-MAC Calculation                *
*************************************************/
void SSL3_MAC::final_result(byte mac[])
   {
   hash->final(mac);
   hash->update(o_key, o_key.size());
   hash->update(mac, OUTPUT_LENGTH);
   hash->final(mac);
   hash->update(i_key, i_key.size());
   }

/*************************************************
* SSL3-MAC Key Schedule                          *
*************************************************/
void SSL3_MAC::key(const byte key[], u32bit length)
   {
   hash->clear();
   i_key.set(0x36);
   o_key.set(0x5C);

   i_key.copy(key, length);
   o_key.copy(key, length);
   hash->update(i_key, i_key.size());
   }

/*************************************************
* Clear memory of sensitive data                 *
*************************************************/
void SSL3_MAC::clear() throw()
   {
   hash->clear();
   i_key.clear();
   o_key.clear();
   }

/*************************************************
* Return the name of this type                   *
*************************************************/
std::string SSL3_MAC::name() const
   {
   return "SSL3-MAC(" + hash->name() + ")";
   }

/*************************************************
* Return a clone of this object                  *
*************************************************/
MessageAuthenticationCode* SSL3_MAC::clone() const
   {
   return new SSL3_MAC(hash->name());
   }

/*************************************************
* SSL3-MAC Constructor                           *
*************************************************/
SSL3_MAC::SSL3_MAC(const std::string& hash_name) :
   MessageAuthenticationCode(output_length_of(hash_name), hash->OUTPUT_LENGTH),
   hash(get_hash(hash_name))
   {
   if(hash->name() != "MD5" && hash->name() != "SHA-160")
      throw Invalid_Argument("SSL3-MAC cannot be used with " + hash->name());

   const u32bit INNER_HASH_LENGTH = (hash->name() == "MD5") ? 48 : 40;

   i_key.create(INNER_HASH_LENGTH);
   o_key.create(INNER_HASH_LENGTH);
   }

}

==> Botan-1.3.3.tgz: Botan-1.3.3/src/ssl3_mac.cpp <==
/*************************************************
* SSL3-MAC Source File                           *
* (C) 1999-2003 The Botan Project                *
*************************************************/

#include <botan/ssl3_mac.h>
#include <botan/lookup.h>

namespace Botan {

/*************************************************
* Update a SSL3-MAC Calculation                  *
*************************************************/
void SSL3_MAC::add_data(const byte input[], u32bit length)
   {
   hash->update(input, length);
   }

/*************************************************
* Finalize a SSL3-MAC Calculation                *
*************************************************/
void SSL3_MAC::final_result(byte mac[])
   {
   hash->final(mac);
   hash->update(o_key, o_key.size());
   hash->update(mac, OUTPUT_LENGTH);
   hash->final(mac);
   hash->update(i_key, i_key.size());
   }

/*************************************************
* SSL3-MAC Key Schedule                          *
*************************************************/
void SSL3_MAC::key(const byte key[], u32bit length)
   {
   hash->clear();
   i_key.set(0x36);
   o_key.set(0x5C);

   i_key.copy(key, length);
   o_key.copy(key, length);
   hash->update(i_key, i_key.size());
   }

/*************************************************
* Clear memory of sensitive data                 *
*************************************************/
void SSL3_MAC::clear() throw()
   {
   hash->clear();
   i_key.clear();
   o_key.clear();
   }

/*************************************************
* Return the name of this type                   *
*************************************************/
std::string SSL3_MAC::name() const
   {
   return "SSL3-MAC(" + hash->name() + ")";
   }

/*************************************************
* Return a clone of this object                  *
*************************************************/
MessageAuthenticationCode* SSL3_MAC::clone() const
   {
   return new SSL3_MAC(hash->name());
   }

/*************************************************
* SSL3-MAC Constructor                           *
*************************************************/
SSL3_MAC::SSL3_MAC(const std::string& hash_name) :
   MessageAuthenticationCode(output_length_of(hash_name),
                             output_length_of(hash_name)),
   hash(get_hash(hash_name))
   {
   if(hash->name() != "MD5" && hash->name() != "SHA-160")
      throw Invalid_Argument("SSL3-MAC cannot be used with " + hash->name());

   const u32bit INNER_HASH_LENGTH = (hash->name() == "MD5") ? 64 : 60;

   i_key.create(INNER_HASH_LENGTH);
   o_key.create(INNER_HASH_LENGTH);
   }

}
